-- cuboid (drill-down next cuboid)

SELECT PC.product_family, S.store_country, T.quarter, SUM(store_sales) AS sales_sum
 FROM sales_fact AS F, product AS P, product_class as PC, time_by_day AS T, store AS S
 WHERE F.product_id = P.product_id 
   AND P.product_class_id = PC.product_class_id
   AND F.time_id = T.time_id 
   AND F.store_id = S.store_id
 GROUP BY PC.product_family, S.store_country, T.quarter 
 ORDER BY PC.product_family, S.store_country, T.quarter

-- rollup previous cuboid

SELECT S.store_country, T.quarter, SUM(store_sales) AS sales_sum
 FROM sales_fact AS F, product AS P, product_class as PC, time_by_day AS T, store AS S
 WHERE F.time_id = T.time_id 
   AND F.store_id = S.store_id
 GROUP BY S.store_country, T.quarter 
 ORDER BY  S.store_country, T.quarter
 
-- data cube - extended cube by considering all subset of values

SELECT PC.product_family, S.store_country, T.quarter, SUM(store_sales) AS sales_sum
 FROM sales_fact AS F, product AS P, product_class as PC, time_by_day AS T, store AS S
 WHERE F.product_id = P.product_id 
   AND P.product_class_id = PC.product_class_id
   AND F.time_id = T.time_id 
   AND F.store_id = S.store_id
 GROUP BY CUBE(PC.product_family, S.store_country, T.quarter) 
 ORDER BY PC.product_family, S.store_country, T.quarter 
 
-- data cube with ROLLUP - subsequences

SELECT PC.product_family, S.store_country, T.quarter, SUM(store_sales) AS sales_sum
 FROM sales_fact AS F, product AS P, product_class as PC, time_by_day AS T, store AS S
 WHERE F.product_id = P.product_id 
   AND P.product_class_id = PC.product_class_id
   AND F.time_id = T.time_id 
   AND F.store_id = S.store_id
 GROUP BY ROLLUP(PC.product_family, S.store_country, T.quarter) 
 ORDER BY PC.product_family, S.store_country, T.quarter 
 
-- data cube with slice

SELECT PC.product_family, S.store_country, T.quarter, SUM(store_sales) AS sales_sum
 FROM sales_fact AS F, product AS P, product_class as PC, time_by_day AS T, store AS S
 WHERE F.product_id = P.product_id 
   AND P.product_class_id = PC.product_class_id
   AND F.time_id = T.time_id 
   AND F.store_id = S.store_id
   AND T.the_day = 'Monday'
 GROUP BY PC.product_family, S.store_country, T.quarter
 ORDER BY PC.product_family, S.store_country, T.quarter



 --COLUMNSTORE INDEXES

 SELECT PC.product_family, S.store_country, T.quarter, SUM(store_sales) AS sales_sum
 FROM salse_fact AS F, product AS P, product_class as PC, time_by_day AS T, store AS S
 WHERE F.product_id = P.product_id 
   AND P.product_class_id = PC.product_class_id
   AND F.time_id = T.time_id 
   AND F.store_id = S.store_id
 GROUP BY PC.product_family, S.store_country, T.quarter 
 ORDER BY PC.product_family, S.store_country, T.quarter
 
 -- SEE the execution plan cost of sales_fact access
 
 SELECT * INTO sales_fact_copy from sales_fact

 CREATE CLUSTERED COLUMNSTORE INDEX cci_sales ON sales_fact_copy

  -- SEE the execution plan cost of sales_fact_copy access

 SELECT PC.product_family, S.store_country, T.quarter, SUM(store_sales) AS sales_sum
 FROM sales_fact_copy AS F, product AS P, product_class as PC, time_by_day AS T, store AS S
 WHERE F.product_id = P.product_id 
   AND P.product_class_id = PC.product_class_id
   AND F.time_id = T.time_id 
   AND F.store_id = S.store_id
 GROUP BY PC.product_family, S.store_country, T.quarter 
 ORDER BY PC.product_family, S.store_country, T.quarter

-- analytic SQL
-- show product_family, store_country, time quarter, sales, row number wrt product_family, rank wrt store_country
 
WITH SubQuery AS
(
SELECT PC.product_family, S.store_country, T.quarter, SUM(store_sales) AS sales_sum
 FROM sales_fact AS F, product AS P, product_class as PC, time_by_day AS T, store AS S
 WHERE F.product_id = P.product_id 
   AND P.product_class_id = PC.product_class_id
   AND F.time_id = T.time_id 
   AND F.store_id = S.store_id
 GROUP BY PC.product_family, S.store_country, T.quarter 
 )
SELECT *, 
		row_number() OVER(PARTITION BY product_family ORDER BY sales_sum DESC) As RowNumber,
		dense_rank() OVER(PARTITION BY store_country ORDER BY sales_sum DESC) As DenseRank
FROM SubQuery
ORDER BY store_country, sales_sum DESC
--ORDER BY product_family, sales_sum DESC
--ORDER BY product_family, store_country, quarter

--Alternative solution

SELECT PC.product_family, S.store_country, T.quarter, SUM(store_sales) AS sales_sum,
row_number() OVER(PARTITION BY PC.product_family ORDER BY SUM(store_sales) DESC) As RowNumber,
dense_rank() OVER(PARTITION BY S.store_country ORDER BY SUM(store_sales) DESC) As DenseRank
 FROM sales_fact AS F, product AS P, product_class as PC, time_by_day AS T, store AS S
 WHERE F.product_id = P.product_id 
   AND P.product_class_id = PC.product_class_id
   AND F.time_id = T.time_id 
   AND F.store_id = S.store_id
 GROUP BY PC.product_family, S.store_country, T.quarter 
ORDER BY store_country, sales_sum DESC
  


-- show product_family, store_country, time quarter, ordered by sales in the group and with percentage of sales over the total of the group

WITH SubQuery AS
(
SELECT PC.product_family, S.store_country, T.quarter, 
 dense_rank() over (partition by PC.product_family order by SUM(store_sales)) as [rank], 
 count(*) over (partition by PC.product_family) as n, 
 SUM(store_sales) AS sales_sum
 FROM sales_fact AS F, product AS P, product_class as PC, time_by_day AS T, store AS S
 WHERE F.product_id = P.product_id 
   AND P.product_class_id = PC.product_class_id
   AND F.time_id = T.time_id 
   AND F.store_id = S.store_id
   AND T.the_day = 'Monday'
 GROUP BY PC.product_family, S.store_country, T.quarter
 
 )
SELECT *, sales_sum/sum(sales_sum) OVER(PARTITION BY product_family) AS PercFamily
FROM SubQuery

-- show product_family, store_country, time quarter, sales, running sales
 
WITH SubQuery AS
(
SELECT PC.product_family, S.store_country, T.quarter, SUM(store_sales) AS sales_sum
 FROM sales_fact AS F, product AS P, product_class as PC, time_by_day AS T, store AS S
 WHERE F.product_id = P.product_id 
   AND P.product_class_id = PC.product_class_id
   AND F.time_id = T.time_id 
   AND F.store_id = S.store_id
 GROUP BY PC.product_family, S.store_country, T.quarter 
 )
SELECT *, 
		sum(sales_sum) OVER(PARTITION BY product_family, store_country ORDER BY quarter ROWS UNBOUNDED PRECEDING ) As RunningSales
FROM SubQuery
ORDER BY product_family, store_country, quarter

-- show product_family, store_country, time quarter, sales, and first, lats, previous, next values
 
WITH SubQuery AS
(
SELECT PC.product_family, S.store_country, T.quarter, SUM(store_sales) AS sales_sum
 FROM sales_fact AS F, product AS P, product_class as PC, time_by_day AS T, store AS S
 WHERE F.product_id = P.product_id 
   AND P.product_class_id = PC.product_class_id
   AND F.time_id = T.time_id 
   AND F.store_id = S.store_id
 GROUP BY PC.product_family, S.store_country, T.quarter 
 )
SELECT *, 
		first_value(sales_sum) OVER(PARTITION BY product_family, store_country ORDER BY quarter) As FirstValue,
		last_value(sales_sum) OVER(PARTITION BY product_family ORDER BY store_country) As LastValue,
		lag(sales_sum, 1, 0) OVER(PARTITION BY product_family, store_country ORDER BY quarter) As PreviousValue,
		lead(sales_sum, 1, 0) OVER(PARTITION BY product_family, store_country ORDER BY quarter) As NextValue
FROM SubQuery
ORDER BY product_family, store_country, quarter


-- materialized views
 
 -- 1. create view
 CREATE VIEW dbo.vStoreTime -- same owner of reference table schema
 WITH SCHEMABINDING -- binds the references table schema
 AS
 SELECT S.store_country, T.quarter, SUM(store_sales) AS sales, COUNT_BIG(*) as n -- COUNT_BIG instead of COUNT is required in materialized view (SQL Server restriction)
 FROM dbo.sales_fact AS F, dbo.time_by_day AS T, dbo.store AS S
 WHERE F.time_id = T.time_id 
   AND F.store_id = S.store_id
 GROUP BY S.store_country, T.quarter 

 
 -- 2. normal select -> view plan

 SELECT S.store_country, T.quarter, SUM(store_sales) AS sales, COUNT_BIG(*) as n
 FROM dbo.sales_fact AS F, dbo.time_by_day AS T, dbo.store AS S
 WHERE F.time_id = T.time_id 
   AND F.store_id = S.store_id
 GROUP BY S.store_country, T.quarter 

 -- 3. materialize view (actually, a materialized index)
 
 CREATE UNIQUE CLUSTERED INDEX IDX_V1 
    ON dbo.vStoreTime (store_country, quarter);

-- 4. same select -> view plan

 SELECT S.store_country, T.quarter, SUM(store_sales) AS sales, COUNT_BIG(*) as n
 FROM dbo.sales_fact AS F, dbo.time_by_day AS T, dbo.store AS S
 WHERE F.time_id = T.time_id 
   AND F.store_id = S.store_id
 GROUP BY S.store_country, T.quarter 

-- 5. rollup -> view plan
 SELECT S.store_country, SUM(store_sales) AS sales
 FROM dbo.sales_fact AS F, dbo.time_by_day AS T, dbo.store AS S
 WHERE F.time_id = T.time_id 
   AND F.store_id = S.store_id
 GROUP BY S.store_country 

-- 6. aggregate by cube ->view plan
 SELECT S.store_country, T.quarter, SUM(store_sales) AS sales
 FROM dbo.sales_fact AS F, dbo.time_by_day AS T, dbo.store AS S
 WHERE F.time_id = T.time_id 
   AND F.store_id = S.store_id
 GROUP BY CUBE(S.store_country, T.quarter)
 
 